# -*- coding: utf-8 -*-
from __future__ import print_function
from collections import OrderedDict
from shutil import copytree, ignore_patterns, rmtree
import sys, os, argparse, codecs, yaml

from itunesfslib.helpers import *
from itunesfslib.humanyaml import *

master_locale_name ='en-US'
chars_count_max = 100
filenames = {
    'config_app'            :'config_app.yaml',
    'config_local'          :'config-local.yaml',
    'description'           :'description.txt',
    'keywords'              :'keywords.txt',
    'version_whats_new'     :'whats_new.txt',
    'screenshots'           :'screenshots'
    }
required_asset_types = [
    'config_local',
    'description',
    'keywords'
    ]
# **************************************************
def convert(path, outpath=''):

    if not outpath:
        outpath     = path
        copyfiles   = False
    else:
        copyfiles   = True
    #open app configuration file
    filepath = os.path.join(path,filenames['config_app'])
    if not os.path.isfile(filepath):
        myexit('The app configuration was not found in location {}'.format(filepath))
    stream = open(filepath, "r", encoding='utf-8-sig').read()
    app_configuration = yaml.load(stream)

    #find versions
    versions = []
    for vdir in next(os.walk(path))[1]:
        if vdir.endswith('.itmsp'):continue
        #find locales
        master_locale = {}
        master_locale_config_local = {}
        locales = []
        ldirs = next(os.walk(os.path.join(path,vdir)))[1]
        if master_locale_name not in ldirs:
            myexit('No files found for the default locale ({}) of v{}'.format(master_locale_name, vdir))
        ldirs.insert(0, ldirs.pop(ldirs.index(master_locale_name))) 
        for ldir in ldirs:
            #find assets
            locale = OrderedDict();
            locale.update({'name': ldir})
            asset_types_copied = []
            for asset_type in ['config_local', 'description', 'keywords', 'version_whats_new', 'screenshots']:
                filepath = os.path.join(path,vdir,ldir,filenames[asset_type]);
                if not os.path.exists(filepath):
                    if ldir == master_locale_name:
                        if asset_type in required_asset_types:
                            myexit('The {} was not found in location {}'.format(asset_type, filepath))
                        else:
                            continue
                    else:                           
                        if asset_type in master_locale:
                            asset_types_copied.append(asset_type)
                            locale.update({asset_type : master_locale[asset_type]})
                        elif asset_type == 'config_local':
                            asset_types_copied.append(asset_type)
                            locale.update(master_locale_config_local)
                        continue

                if asset_type == 'config_local':
                    stream = codecs.open(filepath, 'r', encoding='utf-8-sig')
                    config_local_dic = yaml.load(stream)
                    locale.update(config_local_dic)
                    if ldir == master_locale_name:
                        master_locale_config_local = config_local_dic
                elif asset_type == 'keywords':
                    keywords_text = codecs.open(filepath, 'r', encoding="utf-8-sig").read()
                    keywords = [keyword.strip() for keyword in keywords_text.split(",")]
                    asset = keywords
                    chars_count = len(keywords)-1 + sum([len(keyword) for keyword in keywords])
                    if chars_count > chars_count_max:
                        printwarning('The maximum keyword character count was exceeded in {} locale of v{}'.format(ldir, vdir))
                    locale.update({asset_type : asset})
                elif asset_type == 'description':
                   asset = open(filepath, 'r', encoding='utf-8-sig').read()
                   locale.update({asset_type : literal(asset)})
                elif asset_type == 'version_whats_new':
                    asset = codecs.open(filepath, 'r', encoding='utf-8-sig').read()
                    locale.update({asset_type : literal(asset)})
                elif asset_type == 'screenshots':
                    devices_screenshots = {}
                    for device in ['iphone_4in', 'iphone_3.5in', 'ipad']:
                        device_filepath = os.path.join(filepath, device)
                        if not os.path.exists(device_filepath):
                            continue
                        screenshots = []
                        relative_path_to_output = os.path.relpath(device_filepath, path)
                        for screenshot in next(os.walk(device_filepath))[2]:
                            if not screenshot.startswith('.'):
                                screenshots.append(os.path.join(relative_path_to_output, screenshot))
                        devices_screenshots.update({device : screenshots})
                    if not devices_screenshots:
                        myexit("No screenshots were found for {} locale of v{}".format(ldir, vdir))
                    if copyfiles:
                        copypath = os.path.join(outpath, vdir, ldir, filenames[asset_type])
                        if os.path.exists(copypath):
                            rmtree(copypath)
                        copytree(filepath, copypath, ignore=ignore_patterns('.*'))
                    locale.update({asset_type : devices_screenshots})                             
            locales.append(locale)
            if ldir == master_locale_name:
                master_locale = locale
        version = OrderedDict();
        version.update({'name'       : vdir})
        version.update({'locales'    : locales})
        versions.append(version)

    app_configuration.update({'versions' : versions})


    #convert app_configuration Python object to YAML
    yaml_data = dump_humanised(app_configuration)
    if not os.path.exists(outpath):
        os.makedirs(outpath)
    with open(os.path.join(outpath, 'output.yaml'), 'w', encoding='utf-8-sig') as outfile:
        outfile.write(yaml_data)



# -*- coding: utf-8 -*-
from __future__ import print_function
import sys
from xtermcolor import colorize

def printsuccess(objs):
    print(colorize(objs, ansi=2), file=sys.stdout)
def printwarning(objs):
    print(colorize("WARNING: "+objs, ansi=9), file=sys.stdout)
def myexit(objs):
    print(colorize("EXITING: "+objs, ansi=1),file=sys.stderr)
    sys.exit(1)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from __future__ import print_function
import sys
import os
import subprocess
import argparse
import shutil
import colorama
from itunesfslib import converter
from itunesfslib.helpers import *



def main():
    colorama.init()
    parser = argparse.ArgumentParser(
        description='Generates a YAML file or an iTunes Store Transporter package (.itmsp) from a folder\'s assets',
        epilog="Author: Petros Douvantzis @ Evil Window Dod")
    parser.add_argument('inputpath',
                        help='path to the asset folder')
    parser.add_argument('-o', metavar='outputpath', dest='outputpath',
                        help='path to output folder')
##    parser.add_argument('--package', metavar='package', action='store_const', const=True, default=False,
##                        help='Generate .itmsp package (default: generate YAML file)')
    parser.add_argument('-t', metavar='type', dest='type', choices=["YAML","itmsp"], default="itmsp",
                        help="the type of the output: [YAML, itmsp]. (default is itmsp)")
    
    #sys.argv = [sys.argv[0]] + '../Horizon -out la'.split()#overide input from command line

    # parse arguments
    try:
        args = parser.parse_args()
    except:
        print("****************")
        parser.print_help()
        print("****************")
        sys.exit(1)
        
    inputpath = os.path.normpath(args.inputpath)
    if not args.outputpath:
        outputpath = None
        savepath = inputpath
    else:
        outputpath = os.path.normpath(args.outputpath)
        savepath = outputpath
        if os.path.abspath(inputpath) == os.path.abspath(outputpath):
            myexit('Input path can not be the same as output path')

    # parse folder and generate "output.yaml"
    converter.convert(inputpath, outputpath)
    printsuccess("Succesfully created \"output.yaml\" file at {}".format(os.path.abspath(savepath)))
    if args.type == 'YAML':
        return
    
    # convert "output.yaml" to a .itmsp package
    executable_path = shutil.which('itmsp')
    if not executable_path:
        myexit("""\"itmsp\" is not installed. \n\n"""
               "In order to convert the generated YAML file to .itmsp package,"
               """you have to install itmsp from https://github.com/colinhumber/itunes_transporter_generator.\n\n"""
               """If you have ruby installed, run:\n   \"$ gem install itunes_transporter_generator\"""")
    os.chdir(savepath)
    success = not subprocess.call([executable_path, 'package', '-i', 'output.yaml'])
    if not success:
        myexit('Failed to convert output.yaml to .itmsp package')

if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-
from setuptools import setup
import sys

#pandoc -o README.rst README.md

if sys.version_info < (3, 0):
    sys.stdout.write("Sorry, requires Python 3.x, not Python 2.x\n")
    sys.exit(1)

setup(
    name='itunesfs',
    version='1.0.0.6',
    description='Script (python3 atm) for generating an iTunes Connect package (.itmsp) straight from your file system',
    long_description=open('README.rst', "r", encoding='utf-8-sig').read(),
    author='Petros Douvantzis',
    author_email='petrakeas@gmail.com',   
    packages=['itunesfslib'],
    entry_points = {
        'console_scripts': ['itunesfs=itunesfslib.command_line:main'],
    },
    url='https://github.com/evilwindowdog/itunesfs',
    license='MIT',
    install_requires=[
    "pyyaml",
    "xtermcolor",
    "colorama"
    ],
    keywords=['itunesconnect', 'localization', 'yaml','apple','itunes','iTMStransporter','iOS'],
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
    ]
)
